package ru.dbotthepony.mc.otm.container

import net.minecraft.world.Container
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.kommons.util.Delegate
import ru.dbotthepony.mc.otm.core.isNotEmpty

/**
 * While this somewhat similar to [net.minecraft.world.inventory.Slot], this slot is not meant
 * for Player interaction.
 */
interface IContainerSlot : Delegate<ItemStack> {
	val slot: Int
	val container: Container

	fun setChanged()
	var item: ItemStack

	operator fun component1() = slot
	operator fun component2() = item

	/**
	 * Max stack size regardless of item
	 *
	 * Prefer to use ItemStack version instead
	 */
	val maxStackSize: Int

	/**
	 * Max amount of [item] that can be stored in this slot.
	 *
	 * This may be larger or smaller than value returned [ItemStack.getMaxStackSize],
	 * and as such returned value by this method should be ground truth
	 */
	fun maxStackSize(item: ItemStack): Int {
		return maxStackSize.coerceAtMost(item.maxStackSize)
	}

	fun remove(): ItemStack
	fun remove(count: Int): ItemStack

	override fun get(): ItemStack {
		return item
	}

	override fun accept(t: ItemStack) {
		item = t
	}

	val isEmpty: Boolean
		get() = item.isEmpty

	val isNotEmpty: Boolean
		get() = item.isNotEmpty

	class Simple(override val slot: Int, override val container: Container) : IContainerSlot {
		override fun setChanged() {
			container.setChanged()
		}

		override var item: ItemStack
			get() = container[slot]
			set(value) { container[slot] = value }
		override val maxStackSize: Int
			get() = container.maxStackSize

		override fun remove(count: Int): ItemStack {
			return container.removeItem(slot, count)
		}

		override fun remove(): ItemStack {
			return container.removeItemNoUpdate(slot)
		}

		override fun maxStackSize(item: ItemStack): Int {
			return maxStackSize.coerceAtMost(item.maxStackSize)
		}
	}
}
